@@ -28,6 +28,7 @@
 #include "goaprovider.h"
 #include "goaexchangeprovider.h"
 #include "goaeditablelabel.h"
+#include "goaspinnerbutton.h"
 #include "goautils.h"
 
 /**
@@ -361,7 +362,7 @@ typedef struct
 
   GtkWidget *cluebar;
   GtkWidget *cluebar_label;
-  GtkWidget *spinner;
+  GtkWidget *spinner_button;
 
   GtkWidget *email_address;
   GtkWidget *password;
@@ -436,45 +437,38 @@ on_email_address_or_password_changed (GtkEditable *editable, gpointer user_data)
 }
 
 static void
-create_account_details_ui (GtkBox *vbox, gboolean new_account, AddAccountData *data)
+create_account_details_ui (GoaProvider    *provider,
+                           GtkDialog      *dialog,
+                           GtkBox         *vbox,
+                           gboolean        new_account,
+                           AddAccountData *data)
 {
-  GtkWidget *header_grid;
+  GtkWidget *grid0;
   GtkWidget *grid1;
   GtkWidget *grid2;
   GtkWidget *hbox;
-  GtkWidget *label;
-  gchar *markup;
   gint width;
 
+  goa_utils_set_dialog_title (provider, dialog, new_account);
+
+  grid0 = gtk_grid_new ();
+  gtk_container_set_border_width (GTK_CONTAINER (grid0), 5);
+  gtk_widget_set_margin_bottom (grid0, 6);
+  gtk_orientable_set_orientation (GTK_ORIENTABLE (grid0), GTK_ORIENTATION_VERTICAL);
+  gtk_grid_set_row_spacing (GTK_GRID (grid0), 12);
+  gtk_container_add (GTK_CONTAINER (vbox), grid0);
+
   data->cluebar = gtk_info_bar_new ();
   gtk_info_bar_set_message_type (GTK_INFO_BAR (data->cluebar), GTK_MESSAGE_ERROR);
+  gtk_widget_set_hexpand (data->cluebar, TRUE);
   gtk_widget_set_no_show_all (data->cluebar, TRUE);
-  gtk_box_pack_start (GTK_BOX (vbox), data->cluebar, FALSE, FALSE, 0);
+  gtk_container_add (GTK_CONTAINER (grid0), data->cluebar);
 
   data->cluebar_label = gtk_label_new ("");
   gtk_label_set_line_wrap (GTK_LABEL (data->cluebar_label), TRUE);
   gtk_container_add (GTK_CONTAINER (gtk_info_bar_get_content_area (GTK_INFO_BAR (data->cluebar))),
                      data->cluebar_label);
 
-  header_grid = gtk_grid_new ();
-  gtk_orientable_set_orientation (GTK_ORIENTABLE (header_grid), GTK_ORIENTATION_HORIZONTAL);
-  gtk_box_pack_start (vbox, header_grid, FALSE, FALSE, 0);
-
-  label = gtk_label_new (NULL);
-  gtk_widget_set_hexpand (label, TRUE);
-  markup = g_strconcat ("<b>",
-                        (new_account) ? _("New Microsoft Exchange Account") : _("Microsoft Exchange Account"),
-                        "</b>",
-                        NULL);
-  gtk_label_set_markup (GTK_LABEL (label), markup);
-  g_free (markup);
-  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
-  gtk_container_add (GTK_CONTAINER (header_grid), label);
-
-  data->spinner = gtk_spinner_new ();
-  gtk_widget_set_no_show_all (data->spinner, TRUE);
-  gtk_container_add (GTK_CONTAINER (header_grid), data->spinner);
-
   grid1 = gtk_grid_new ();
   gtk_orientable_set_orientation (GTK_ORIENTABLE (grid1), GTK_ORIENTATION_VERTICAL);
   gtk_grid_set_row_spacing (GTK_GRID (grid1), 12);
@@ -487,7 +481,7 @@ create_account_details_ui (GtkBox *vbox, gboolean new_account, AddAccountData *d
   gtk_box_set_homogeneous (GTK_BOX (hbox), FALSE);
   gtk_box_pack_start (GTK_BOX (hbox), grid1, FALSE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX (hbox), grid2, TRUE, TRUE, 0);
-  gtk_box_pack_start (vbox, hbox, FALSE, FALSE, 0);
+  gtk_container_add (GTK_CONTAINER (grid0), hbox);
 
   add_entry (grid1, grid2, _("_E-mail"), &data->email_address);
   add_entry (grid1, grid2, _("_Password"), &data->password);
@@ -496,7 +490,7 @@ create_account_details_ui (GtkBox *vbox, gboolean new_account, AddAccountData *d
       data->expander = gtk_expander_new_with_mnemonic (_("_Custom"));
       gtk_expander_set_expanded (GTK_EXPANDER (data->expander), FALSE);
       gtk_expander_set_resize_toplevel (GTK_EXPANDER (data->expander), TRUE);
-      gtk_box_pack_start (vbox, data->expander, FALSE, FALSE, 0);
+      gtk_container_add (GTK_CONTAINER (grid0), data->expander);
 
       grid1 = gtk_grid_new ();
       gtk_orientable_set_orientation (GTK_ORIENTABLE (grid1), GTK_ORIENTATION_VERTICAL);
@@ -523,7 +517,8 @@ create_account_details_ui (GtkBox *vbox, gboolean new_account, AddAccountData *d
   g_signal_connect (data->email_address, "changed", G_CALLBACK (on_email_address_or_password_changed), data);
   g_signal_connect (data->password, "changed", G_CALLBACK (on_email_address_or_password_changed), data);
 
-  gtk_dialog_add_button (data->dialog, GTK_STOCK_CONNECT, GTK_RESPONSE_OK);
+  data->spinner_button = goa_spinner_button_new_from_stock (GTK_STOCK_CONNECT);
+  gtk_dialog_add_action_widget (data->dialog, data->spinner_button, GTK_RESPONSE_OK);
   gtk_dialog_set_default_response (data->dialog, GTK_RESPONSE_OK);
   gtk_dialog_set_response_sensitive (data->dialog, GTK_RESPONSE_OK, FALSE);
 
@@ -552,8 +547,7 @@ autodiscover_cb (GObject *source_object, GAsyncResult *res, gpointer user_data)
 
   goa_ews_client_autodiscover_finish (client, res, &data->error);
   g_main_loop_quit (data->loop);
-  gtk_spinner_stop (GTK_SPINNER (data->spinner));
-  gtk_widget_hide (data->spinner);
+  goa_spinner_button_stop (GOA_SPINNER_BUTTON (data->spinner_button));
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -585,7 +579,7 @@ add_account (GoaProvider    *provider,
   data.dialog = dialog;
   data.error = NULL;
 
-  create_account_details_ui (vbox, TRUE, &data);
+  create_account_details_ui (provider, dialog, vbox, TRUE, &data);
   gtk_widget_show_all (GTK_WIDGET (vbox));
 
   ews_client = goa_ews_client_new ();
@@ -625,13 +619,11 @@ add_account (GoaProvider    *provider,
                                NULL,
                                autodiscover_cb,
                                &data);
-  gtk_spinner_start (GTK_SPINNER (data.spinner));
-  gtk_widget_show (data.spinner);
+  goa_spinner_button_start (GOA_SPINNER_BUTTON (data.spinner_button));
   g_main_loop_run (data.loop);
 
   if (data.error != NULL)
     {
-      GtkWidget *button;
       gchar *markup;
 
       markup = g_strdup_printf ("<b>%s:</b> %s",
@@ -642,8 +634,7 @@ add_account (GoaProvider    *provider,
       gtk_label_set_markup (GTK_LABEL (data.cluebar_label), markup);
       g_free (markup);
 
-      button = gtk_dialog_get_widget_for_response (data.dialog, GTK_RESPONSE_OK);
-      gtk_button_set_label (GTK_BUTTON (button), _("_Try Again"));
+      goa_spinner_button_set_label (GOA_SPINNER_BUTTON (data.spinner_button), _("_Try Again"));
       gtk_expander_set_expanded (GTK_EXPANDER (data.expander), TRUE);
       gtk_widget_set_no_show_all (data.cluebar, FALSE);
       gtk_widget_show_all (data.cluebar);
@@ -748,7 +739,7 @@ refresh_account (GoaProvider    *provider,
   data.dialog = GTK_DIALOG (dialog);
   data.error = NULL;
 
-  create_account_details_ui (GTK_BOX (vbox), FALSE, &data);
+  create_account_details_ui (provider, GTK_DIALOG (dialog), GTK_BOX (vbox), FALSE, &data);
 
   account = goa_object_peek_account (object);
   email_address = goa_account_get_presentation_identity (account);
@@ -784,8 +775,7 @@ refresh_account (GoaProvider    *provider,
                                NULL,
                                autodiscover_cb,
                                &data);
-  gtk_spinner_start (GTK_SPINNER (data.spinner));
-  gtk_widget_show (data.spinner);
+  goa_spinner_button_start (GOA_SPINNER_BUTTON (data.spinner_button));
   g_main_loop_run (data.loop);
 
   if (data.error != NULL)
