@@ -29,6 +29,7 @@
 #include "goaprovider.h"
 #include "goakerberosprovider.h"
 #include "goaeditablelabel.h"
+#include "goaspinnerbutton.h"
 #include "goautils.h"
 #include "goaidentity.h"
 
@@ -90,7 +91,7 @@ typedef struct
 
   GtkWidget *cluebar;
   GtkWidget *cluebar_label;
-  GtkWidget *spinner;
+  GtkWidget *spinner_button;
 
   GtkWidget *username;
   GtkWidget *realm_entry;
@@ -967,42 +968,31 @@ on_object_manager_ensured_for_getting_realms (GoaKerberosProvider *self,
 
 static void
 create_account_details_ui (GoaKerberosProvider *self,
+                           GtkDialog           *dialog,
                            GtkWidget           *vbox,
                            gboolean             new_account,
                            SignInRequest       *request)
 {
-  GtkWidget *header_grid;
+  GtkWidget *grid0;
   GtkWidget *grid1;
   GtkWidget *grid2;
   GtkWidget *hbox;
-  GtkWidget *label;
-  gchar *markup;
   gint width;
 
-  header_grid = gtk_grid_new ();
-  gtk_orientable_set_orientation (GTK_ORIENTABLE (header_grid), GTK_ORIENTATION_HORIZONTAL);
-  gtk_box_pack_start (GTK_BOX (vbox), header_grid, FALSE, FALSE, 0);
+  goa_utils_set_dialog_title (GOA_PROVIDER (self), dialog, new_account);
 
-  label = gtk_label_new (NULL);
-  gtk_widget_set_hexpand (label, TRUE);
-  markup = g_strconcat ("<b>",
-                        (new_account) ? _("New Enterprise Login (Kerberos)") : _("Enterprise Login (Kerberos)"),
-                        "</b>",
-                        NULL);
-  gtk_label_set_markup (GTK_LABEL (label), markup);
-  g_free (markup);
-  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
-  gtk_misc_set_padding (GTK_MISC (label), 0.0, 12);
-  gtk_container_add (GTK_CONTAINER (header_grid), label);
-
-  request->spinner = gtk_spinner_new ();
-  gtk_widget_set_no_show_all (request->spinner, TRUE);
-  gtk_container_add (GTK_CONTAINER (header_grid), request->spinner);
+  grid0 = gtk_grid_new ();
+  gtk_container_set_border_width (GTK_CONTAINER (grid0), 5);
+  gtk_widget_set_margin_bottom (grid0, 6);
+  gtk_orientable_set_orientation (GTK_ORIENTABLE (grid0), GTK_ORIENTATION_VERTICAL);
+  gtk_grid_set_row_spacing (GTK_GRID (grid0), 12);
+  gtk_container_add (GTK_CONTAINER (vbox), grid0);
 
   request->cluebar = gtk_info_bar_new ();
   gtk_info_bar_set_message_type (GTK_INFO_BAR (request->cluebar), GTK_MESSAGE_ERROR);
+  gtk_widget_set_hexpand (request->cluebar, TRUE);
   gtk_widget_set_no_show_all (request->cluebar, TRUE);
-  gtk_box_pack_start (GTK_BOX (vbox), request->cluebar, FALSE, FALSE, 0);
+  gtk_container_add (GTK_CONTAINER (grid0), request->cluebar);
 
   request->cluebar_label = gtk_label_new ("");
   gtk_label_set_line_wrap (GTK_LABEL (request->cluebar_label), TRUE);
@@ -1021,7 +1011,7 @@ create_account_details_ui (GoaKerberosProvider *self,
   gtk_box_set_homogeneous (GTK_BOX (hbox), FALSE);
   gtk_box_pack_start (GTK_BOX (hbox), grid1, FALSE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX (hbox), grid2, TRUE, TRUE, 0);
-  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
+  gtk_container_add (GTK_CONTAINER (grid0), hbox);
 
   request->realm_store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_OBJECT);
   add_combo_box (grid1,
@@ -1043,7 +1033,8 @@ create_account_details_ui (GoaKerberosProvider *self,
 
   gtk_widget_grab_focus (request->realm_combo_box);
 
-  gtk_dialog_add_button (request->dialog, GTK_STOCK_CONNECT, GTK_RESPONSE_OK);
+  request->spinner_button = goa_spinner_button_new_from_stock (GTK_STOCK_CONNECT);
+  gtk_dialog_add_action_widget (request->dialog, request->spinner_button, GTK_RESPONSE_OK);
   gtk_dialog_set_default_response (request->dialog, GTK_RESPONSE_OK);
   gtk_dialog_set_response_sensitive (request->dialog, GTK_RESPONSE_OK, TRUE);
 
@@ -1332,7 +1323,7 @@ add_account (GoaProvider    *provider,
   request.dialog = dialog;
   request.error = NULL;
 
-  create_account_details_ui (self, GTK_WIDGET (vbox), TRUE, &request);
+  create_account_details_ui (self, dialog, GTK_WIDGET (vbox), TRUE, &request);
   gtk_widget_show_all (GTK_WIDGET (vbox));
 
 start_over:
@@ -1388,7 +1379,10 @@ start_over:
                                 NULL, /* GCancellable* */
                                 (GAsyncReadyCallback) add_account_cb,
                                 &request);
+
+  goa_spinner_button_start (GOA_SPINNER_BUTTON (request.spinner_button));
   g_main_loop_run (request.loop);
+  goa_spinner_button_stop (GOA_SPINNER_BUTTON (request.spinner_button));
   if (request.error != NULL)
     goto out;
 
@@ -1396,18 +1390,16 @@ start_over:
                                                          request.account_object_path));
   account = goa_object_peek_account (object);
 
-  gtk_spinner_start (GTK_SPINNER (request.spinner));
-  gtk_widget_show (request.spinner);
-
   /* After the account is created, try to sign it in
    */
   perform_initial_sign_in (self, object, principal, &request);
 
+  goa_spinner_button_start (GOA_SPINNER_BUTTON (request.spinner_button));
   g_main_loop_run (request.loop);
+  goa_spinner_button_stop (GOA_SPINNER_BUTTON (request.spinner_button));
 
   if (request.error != NULL)
     {
-      GtkWidget *button;
       gchar *markup;
 
       translate_error (&request.error);
@@ -1422,8 +1414,7 @@ start_over:
           gtk_label_set_markup (GTK_LABEL (request.cluebar_label), markup);
           g_free (markup);
 
-          button = gtk_dialog_get_widget_for_response (request.dialog, GTK_RESPONSE_OK);
-          gtk_button_set_label (GTK_BUTTON (button), _("_Try Again"));
+          goa_spinner_button_set_label (GOA_SPINNER_BUTTON (request.spinner_button), _("_Try Again"));
           gtk_widget_set_no_show_all (request.cluebar, FALSE);
           gtk_widget_show_all (request.cluebar);
         }
